/**
 * Copyright (C) 2015 Wasabeef
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var RE = {};

RE.currentSelection = {
    "startContainer": 0,
    "startOffset": 0,
    "endContainer": 0,
    "endOffset": 0};

RE.editor = document.getElementById('editor');

document.addEventListener("selectionchange", function() {
RE.backuprange();
BoldInterface.isBold(document.queryCommandState('bold'));
});

// Initializations
RE.callback = function() {
    window.location.href = "re-callback://" + encodeURIComponent(RE.getHtml());
}

RE.setHtml = function(contents) {
    RE.editor.innerHTML = decodeURIComponent(contents.replace(/\+/g, '%20'));
}

RE.getEncodeHtml = function() {
    window.callback.getEncodeHtml(encodeURIComponent(RE.getHtml()));
}

RE.getHtml = function() {
    return RE.editor.innerHTML;
}

RE.getText = function() {
    return RE.editor.innerText;
}

RE.setBaseTextColor = function(color) {
    RE.editor.style.color  = color;
}

RE.setBaseFontSize = function(size) {
    RE.editor.style.fontSize = size;
}

RE.setPadding = function(left, top, right, bottom) {
  RE.editor.style.paddingLeft = left;
  RE.editor.style.paddingTop = top;
  RE.editor.style.paddingRight = right;
  RE.editor.style.paddingBottom = bottom;
}

RE.setBackgroundColor = function(color) {
    document.body.style.backgroundColor = color;
}

RE.setBackgroundImage = function(image) {
    RE.editor.style.backgroundImage = image;
}

RE.setWidth = function(size) {
    RE.editor.style.minWidth = size;
}

RE.setHeight = function(size) {
    RE.editor.style.height = size;
}

RE.setTextAlign = function(align) {
    RE.editor.style.textAlign = align;
}

RE.setVerticalAlign = function(align) {
    RE.editor.style.verticalAlign = align;
}

RE.setPlaceholder = function(placeholder) {
    RE.editor.setAttribute("placeholder", placeholder);
}

RE.undo = function() {
    document.execCommand('undo', false, null);
}

RE.redo = function() {
    document.execCommand('redo', false, null);
}

RE.setBold = function() {
    document.execCommand('bold', false, null);
}

RE.setItalic = function() {
    document.execCommand('italic', false, null);
}

RE.setSubscript = function() {
    document.execCommand('subscript', false, null);
}

RE.setSuperscript = function() {
    document.execCommand('superscript', false, null);
}

RE.setStrikeThrough = function() {
    document.execCommand('strikeThrough', false, null);
}

RE.setUnderline = function() {
    document.execCommand('underline', false, null);
}

RE.setBullets = function() {
    document.execCommand('InsertUnorderedList', false, null);
}

RE.setNumbers = function() {
    document.execCommand('InsertOrderedList', false, null);
}

RE.setTextColor = function(color) {
    RE.restorerange();
    document.execCommand("styleWithCSS", null, true);
    document.execCommand('foreColor', false, color);
    document.execCommand("styleWithCSS", null, false);
}

RE.setTextBackgroundColor = function(color) {
    RE.restorerange();
    document.execCommand("styleWithCSS", null, true);
    document.execCommand('hiliteColor', false, color);
    document.execCommand("styleWithCSS", null, false);
}

RE.setFontSize = function(fontSize){
    document.execCommand("fontSize", false, fontSize);
}

RE.setHeading = function(heading) {
    document.execCommand('formatBlock', false, '<h'+heading+'>');
}

RE.setIndent = function() {
    document.execCommand('indent', false, null);
}

RE.setOutdent = function() {
    document.execCommand('outdent', false, null);
}

RE.setJustifyLeft = function() {
    document.execCommand('justifyLeft', false, null);
}

RE.setJustifyCenter = function() {
    document.execCommand('justifyCenter', false, null);
}

RE.setJustifyRight = function() {
    document.execCommand('justifyRight', false, null);
}

RE.setBlockquote = function() {
    document.execCommand('formatBlock', false, '<blockquote>');
}

RE.scrollToSelection = function() {
  $('body').scrollTop($(window.getSelection().anchorNode).offset().top);
}

RE.insertImage = function(url, type, id, content, width, height, ext) {
//height为了自适应宽、单位那边加个；号
    var html = '<img style="display:block;width:' + width +  'px;height:' + height + ';px;margin-top:4px;margin-bottom:8px;"  src="' + url + '" data-type="' + type+ '" data-id="' + id +
    '" data-value="' + content;
    if(ext != '') {
    html = html + '" data-ext="' + ext
    }
     html = html + '" />';
    RE.insertHTML(html);
}
RE.insertEmotion = function(url, type, id, content, width, height) {
    var html = '<img style="width:' + width +  'px;height:' + height + 'px;margin-bottom:4px;"  src="' + url + '" data-type="' + type+ '" data-id="' + id +
    '" data-value="' + content +'" />';
    RE.insertHTML(html);
}
RE.insertContact = function(uid,uname,wid) {
var txt = document.createElement('input');
	txt.setAttribute('type','text');
	txt.setAttribute('readonly','readonly');
	txt.setAttribute('style','width:'+wid+"px");
	txt.setAttribute('class','contact');
	txt.setAttribute('data-id',uid);
	txt.setAttribute('value',uname);
    RE.insertHTML(txt.outerHTML);

}

RE.insertHTML = function(html) {
    RE.restorerange();
    document.execCommand('insertHTML', false, html);
}

RE.deleteForward = function() {
    document.execCommand('delete', false, null);
}

function deleteElement(id) {
  var el = RE.editor;
  var range = document.createRange();
  range.selectNode(el);
  var sel = window.getSelection();
  sel.removeAllRanges();
  sel.addRange(range);
  document.execCommand("delete", false, null);
}

RE.insertLink = function(url, title) {
    RE.restorerange();
    var sel = document.getSelection();
    if (sel.toString().length == 0) {
        document.execCommand("insertHTML",false,"<a href='"+url+"'>"+title+"</a>");
    } else if (sel.rangeCount) {
       var el = document.createElement("a");
       el.setAttribute("href", url);
       el.setAttribute("title", title);

       var range = sel.getRangeAt(0).cloneRange();
       range.surroundContents(el);
       sel.removeAllRanges();
       sel.addRange(range);
   }
    RE.callback();
}

RE.setTodo = function(text) {
    var html = '<input type="checkbox" name="'+ text +'" value="'+ text +'"/> &nbsp;';
    document.execCommand('insertHTML', false, html);
}

RE.prepareInsert = function() {
    RE.backuprange();
}

RE.backuprange = function(){
    var selection = window.getSelection();
    if (selection.rangeCount > 0) {
      var range = selection.getRangeAt(0);
      RE.currentSelection = {
          "startContainer": range.startContainer,
          "startOffset": range.startOffset,
          "endContainer": range.endContainer,
          "endOffset": range.endOffset};
    }
}

RE.restorerange = function(){
    var selection = window.getSelection();
    selection.removeAllRanges();
    var range = document.createRange();
    range.setStart(RE.currentSelection.startContainer, RE.currentSelection.startOffset);
    range.setEnd(RE.currentSelection.endContainer, RE.currentSelection.endOffset);
    selection.addRange(range);
}

RE.getRange = function() {
    var selection = window.getSelection();
    if (selection.rangeCount > 0) {
        return selection.getRangeAt(0);
    }
}

RE.getRangeClone = function() {
    var range = RE.getRange();
    if (typeof(range) == "undefined" || range == null) {
        return;
    }
    return range.cloneRange();
}

RE.selectNode = function(node) {
    if (typeof(node) == "undefined" || node == null) {
        return;
    }
    var range = RE.getRange();
    if (typeof(range) == "undefined" || range == null) {
            return;
    }
    range.selectNodeContents(node);
    RE.selectRange(range);
}

RE.selectRange = function(range) {
    if (typeof(range) == "undefined" || range == null) {
        return;
    }
    var sel = window.getSelection();
    sel.removeAllRanges();
    sel.addRange(range);
}

RE.saveRange = function() {
    RE.lastRange = RE.getRangeClone();
}

RE.enabledEditingItems = function(e) {
    var items = [];
    if (document.queryCommandState('bold')) {
        items.push('bold');
    }
    if (document.queryCommandState('italic')) {
        items.push('italic');
    }
    if (document.queryCommandState('subscript')) {
        items.push('subscript');
    }
    if (document.queryCommandState('superscript')) {
        items.push('superscript');
    }
    if (document.queryCommandState('strikeThrough')) {
        items.push('strikeThrough');
    }
    if (document.queryCommandState('underline')) {
        items.push('underline');
    }
    if (document.queryCommandState('insertOrderedList')) {
        items.push('orderedList');
    }
    if (document.queryCommandState('insertUnorderedList')) {
        items.push('unorderedList');
    }
    if (document.queryCommandState('justifyCenter')) {
        items.push('justifyCenter');
    }
    if (document.queryCommandState('justifyFull')) {
        items.push('justifyFull');
    }
    if (document.queryCommandState('justifyLeft')) {
        items.push('justifyLeft');
    }
    if (document.queryCommandState('justifyRight')) {
        items.push('justifyRight');
    }
    if (document.queryCommandState('insertHorizontalRule')) {
        items.push('horizontalRule');
    }
    var formatBlock = document.queryCommandValue('formatBlock');
    if (formatBlock.length > 0) {
        items.push(formatBlock);
    }
    items.push(e.target.getAttribute('data-type'));

    window.location.href = "re-state://" + encodeURI(items.join(','));
}

RE.focus = function() {
    //RE.editor.focus();
    //el=el[0]; //jquery 对象转dom对象
    RE.editor.focus();
    if($.support.msie)
    {
      var range = document.selection.createRange();
      this.last = range;
      range.moveToElementText(RE.editor);
      range.select();
      document.selection.empty(); //取消选中
    }
    else
    {
      var range = document.createRange();
      range.selectNodeContents(RE.editor);
      range.collapse(false);
      var sel = window.getSelection();
      sel.removeAllRanges();
      sel.addRange(range);
    }
}

RE.blurFocus = function() {
    RE.editor.blur();
}

RE.removeFormat = function() {
    execCommand('removeFormat', false, null);
}

/**
 *  从剪切板获取文本
 *  @params e 剪切板对象
 *  @return 是否已处理
 **/
RE.pasteNormalCallback = function(e) {
    if (event.clipboardData || event.originalEvent) {
        var clipboardData = (event.clipboardData || event.originalEvent.clipboardData);
        if(typeof(clipboardData) != "undefined" && clipboardData != null){
            RE.insertText(BoldInterface.getClipboardContent());
            return false;
        }
    }
}

/**
 *  使用透明布局插入方式
 *  代码参照 https://github.com/wbb/wysibb/blob/master/jquery.wysibb.js#L837
 *  @params e 剪切板对象
 *  @return 是否已处理
 **/
RE.pasteSpecialCallback =  function(e) {
    var originScrollTop = $('body').scrollTop();
    if (!RE.editor.pasteBlock) {
        RE.saveRange();
        // append translate div
        RE.pasteBlock = document.createElement("div");
        RE.pasteBlock.setAttribute("style", "opacity:0;");
        RE.pasteBlock.setAttribute("contenteditable", "true");
        RE.pasteBlock.innerHTML = "\uFEFF";
        RE.editor.appendChild(RE.pasteBlock);

        window.setTimeout(function() {
            var data = '<span>'+ RE.pasteBlock.innerHTML+'</span>';
            RE.editor.removeChild(RE.pasteBlock);
            RE.editor.focus();

            RE.selectRange(RE.lastRange);
            alert(data);
            return false;
            window.location.href = "re-check://" + encodeURI(data);

            RE.pasteBlock = null;
            RE.lastRange = null;

            $('body').scrollTop(originScrollTop);
        }, 1);
        RE.selectNode(RE.pasteBlock);
        return true;
    }
    return false;
}
RE.insertText = function(text) {
    RE.restorerange();
    document.execCommand('insertText', false, text);
}

// Event Listeners
RE.editor.addEventListener("input", RE.callback);
RE.editor.addEventListener("keyup", function(e) {
    var KEY_LEFT = 37, KEY_RIGHT = 39;
    if (e.which == KEY_LEFT || e.which == KEY_RIGHT) {
        RE.enabledEditingItems(e);
    }
});
RE.editor.addEventListener("click", function(e) {
     RE.enabledEditingItems(e);
   });

//添加粘贴监听
RE.editor.addEventListener("paste", function(e) {
    e.preventDefault();
    if (event.clipboardData || event.originalEvent) {
        var clipboard = (event.clipboardData || event.originalEvent.clipboardData);
        if(typeof(clipboard) != "undefined" && clipboard != null){
          RE.insertText(BoldInterface.getClipboardContent());
          return false;
        }
    }
});
//替换掉复制粘贴的HTML Tag
RE.filterHTMLTag = function (msg) {
        var msg = msg.replace(/<\/?[^>]*>/g, ''); //去除HTML Tag
        //msg = msg.replace(/[|]*\n/, '') //去除行尾空格
        //msg = msg.replace(/&nbsp;/ig, ''); //去掉nbsp
        return msg;
}
//光标移动到末尾
RE.moveToEnd = function keepLastIndex() {
     var obj = RE.editor;
    if (window.getSelection) {//ie11 10 9 ff safari
        obj.focus(); //解决ff不获取焦点无法定位问题
        var range = window.getSelection();//创建range
        range.selectAllChildren(obj);//range 选择obj下所有子内容
        range.collapseToEnd();//光标移至最后
    }
    else if (document.selection) {//ie10 9 8 7 6 5
        var range = document.selection.createRange();//创建选择对象
        //var range = document.body.createTextRange();
        range.moveToElementText(obj);//range定位到obj
        range.collapse(false);//光标移至最后
        range.select();
    }
}

